__title__ = 'fobi.contrib.plugins.form_handlers.db_store.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'CSV_DELIMITER', 'CSV_QUOTECHAR',
)

CSV_DELIMITER = ','
CSV_QUOTECHAR = '"'
