{% extends "fobi/generic/import_form_entry.html" %}

{% load i18n admin_static %}

{% block stylesheets %}
    <link rel="stylesheet" type="text/css" href="{% static 'admin/css/changelists.css' %}" />
{% endblock stylesheets %}
