{% extends fobi_theme.master_base_template %}

{% block navbar-menu-content %}{% endblock navbar-menu-content %}

{% block content %}{% endblock content %}

{% block sidebar-content %}{% endblock sidebar-content %}
