# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-09-27 22:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('fobi', '0007_auto_20160926_1652'),
    ]

    operations = [
        migrations.CreateModel(
            name='FormWizardHandlerEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('plugin_data', models.TextField(blank=True, null=True, verbose_name='Plugin data')),
                ('plugin_uid', models.CharField(max_length=255, verbose_name='Plugin name')),
                ('form_wizard_entry', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='fobi.FormWizardEntry', verbose_name='Form wizard')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Form wizard handler entry',
                'verbose_name_plural': 'Form wizard handler entries',
            },
        ),
    ]
